#ifndef _TMYLIST_H
#define _TMYLIST_H

#include "TlinkPool.h"

//
// class: TmyList
//
// 		simple link list of integers, the associated
// 		memory manager is included (in TlinkPool), where
// 		if this list is reset (all links deleted), and then
// 		re-inserting new elements will _NOT_ cause new memory
//		to be allocated. TlinkPool will supply the already 
// 		allocated links.
//
class TmyList {

	private:
		TlinkPool	    fMemoryManager;		// link memory manager

		TlinkType 	    *fHeadPointer;	   	// head of the list
		TlinkType       *fCurrentPointer;  	// current link position when
										   	// traversing the list

		int				fListLength;		// Length of this list;


		//
		// supports traversal of the list
		// with the following three functions, 
		// one would typically do:
		//
		//     TmyList  myList;
		//
		//     ...
		//     ...
		//     myList.initListForTraversal();
		//     while (myList.notEndOfList()) {
		//     		//
		//     		//.. list operations ..
		//     		//
		//     		advanceList();
		//     	}
		// 
		void 		initListForTraversal() {
								fCurrentPointer = fHeadPointer;
							}
		bool 		notEndOfList() const {
								return fCurrentPointer != NULL;
							}
		void 		advanceList() {
								fCurrentPointer = fCurrentPointer->nextField();
							}
		

		TlinkType  *removeFirstEntry() {
						TlinkType *entry = fHeadPointer;
						if (fHeadPointer != NULL) {
							fHeadPointer = fHeadPointer->nextField();
							entry->nextField() = NULL;
						}
						return entry;
					}

		TlinkType *breakListToTwoAt(TlinkType *list, int pos);
		void	    mergeSortCombine(TlinkType*&, TlinkType*, TlinkType*);
		void		mergeSortDivide(TlinkType*&,  int);

	public:
		//
		// this is the constructor
		// I am initializing instance 
		// variables in my constructor
		//
		TmyList() :
				fMemoryManager(),			// 
				fListLength(0),
				fHeadPointer(NULL), 		// initialize HeadPointer to NULL
				fCurrentPointer(NULL) {}	// initialize CurrentPointer to NULL


		//
		//
		// simple list insertion/deletion methods
		//
		void 		insertElement(int key) {
						TlinkType *newLink = fMemoryManager.nextFreeLink();
						newLink->keyField() = key;
						newLink->nextField() = fHeadPointer;
						fHeadPointer = newLink;
						fListLength++;
					}
					
		void 		resetList() {
						//
						// reset the list _without_ freeing
						// any of the already allocated memory
						//
						fMemoryManager.resetAllLinks();
						fListLength = 0;
						fHeadPointer = NULL;
					}

		//
		//
		void		sortedListCheck();

		void		printListContent(const char *str, bool shouldPrint);

					//
					// takes the sourceList and copy over
					// the entire list. The resulting list
					// is in reversed order.
		void		duplicateList(TmyList &sourceList);


		void		insertionSort();
		void		mergeSort();
};

#endif
